@@ -4,29 +4,34 @@ from datetime import date
 from pathlib import Path
 from typing import List
 
+import numpy as np
 import toml
 from astropy.io.fits import getdata
 
-from m23.align.alignment import imageAlignment
+from m23.align import image_alignment
 from m23.calibrate.calibration import calibrateImages
 from m23.calibrate.master_calibrate import makeMasterDark
-from m23.combine.combination import imageCombination
+from m23.charts import draw_normfactors_chart
 from m23.constants import (
     ALIGNED_COMBINED_FOLDER_NAME,
     CONFIG_FILE_NAME,
     FLUX_LOGS_COMBINED_FOLDER_NAME,
     INPUT_CALIBRATION_FOLDER_NAME,
-    LOG_FILE_COMBINED_FILENAME_DATE_FORMAT,
     LOG_FILES_COMBINED_FOLDER_NAME,
     M23_RAW_IMAGES_FOLDER_NAME,
     MASTER_DARK_NAME,
     MASTER_FLAT_NAME,
     OUTPUT_CALIBRATION_FOLDER_NAME,
 )
-from m23.extract.extraction import extract_stars
+from m23.extract import extract_stars
+from m23.file.aligned_combined_file import AlignedCombinedFile
+from m23.file.flux_log_combined_file import FluxLogCombinedFile
+from m23.file.log_file_combined_file import LogFileCombinedFile
+from m23.file.raw_image_file import RawImageFile
+from m23.file.reference_log_file import ReferenceLogFile
 from m23.matrix import crop
 from m23.matrix.fill import fillMatrix
-from m23.norm.normalize import normalizeLogFiles
+from m23.norm import normalize_log_files
 from m23.processor.config_loader import Config, ConfigInputNight, validate_file
 from m23.utils import (
     fit_data_from_fit_images,
@@ -43,28 +48,30 @@ from m23.utils import (
 def normalization_helper(
     radii_of_extraction: List[int],
     FLUX_LOGS_COMBINED_OUTPUT_FOLDER: Path,
-    ref_file_path: Path,
-    log_files_to_use: List[Path],
+    reference_log_file: ReferenceLogFile,
+    log_files_to_use: List[LogFileCombinedFile],
+    img_duration: float,
     night_date: date,
 ):
     """
     This is a normalization helper function extracted so that it can be reused by the renormalization script
     """
-    for index, radius in enumerate(radii_of_extraction):
+    for radius in radii_of_extraction:
         logging.info(f"Normalizing for radius of extraction {radius} px")
         RADIUS_FOLDER = FLUX_LOGS_COMBINED_OUTPUT_FOLDER / get_radius_folder_name(radius)
         RADIUS_FOLDER.mkdir(exist_ok=True)  # Create folder if it doesn't exist
         for file in RADIUS_FOLDER.glob("*"):
             if file.is_file():
                 file.unlink()  # Remove each file in the folder
-        logfile_adu_column = 6 + index  # 6th column is the first column with Star ADU
-        normalizeLogFiles(
-            ref_file_path,
+        normalize_log_files(
+            reference_log_file,
             log_files_to_use,
-            logfile_adu_column,
             RADIUS_FOLDER,
+            radius,
+            img_duration,
             night_date,
         )
+    draw_normfactors_chart(log_files_to_use, FLUX_LOGS_COMBINED_OUTPUT_FOLDER.parent)
 
 
 def process_night(night: ConfigInputNight, config: Config, output: Path, night_date: date):
@@ -95,6 +102,7 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
 
     ref_image_path = config["reference"]["image"]
     ref_file_path = config["reference"]["file"]
+    reference_log_file = ReferenceLogFile(ref_file_path)
 
     # Define relevant input folders for the night being processed
     NIGHT_INPUT_FOLDER: Path = night["path"]
@@ -121,8 +129,9 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
 
     # Darks
     darks = fit_data_from_fit_images(get_darks(NIGHT_INPUT_CALIBRATION_FOLDER))
-    if len(crop_region) > 0:
-        darks = [crop(matrix, rows, cols) for matrix in darks]
+    # Ensure that image dimensions are as specified by rows and cols 
+    # If there's extra noise cols or rows, we crop them 
+    darks = [crop(matrix, rows, cols) for matrix in darks] 
     master_dark_data = makeMasterDark(
         saveAs=CALIBRATION_OUTPUT_FOLDER / MASTER_DARK_NAME,
         headerToCopyFromName=next(get_darks(NIGHT_INPUT_CALIBRATION_FOLDER)).absolute(),
@@ -137,8 +146,9 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
         logging.info(f"Using pre-provided masterflat")
     else:
         flats = fit_data_from_fit_images(get_flats(NIGHT_INPUT_CALIBRATION_FOLDER))
-        if len(crop_region) > 0:
-            flats = [crop(matrix, rows, cols) for matrix in flats]
+        # Ensure that image dimensions are as specified by rows and cols 
+        # If there's extra noise cols or rows, we crop them 
+        flats = [crop(matrix, rows, cols) for matrix in flats]
 
         master_flat_data = makeMasterDark(
             saveAs=CALIBRATION_OUTPUT_FOLDER / MASTER_FLAT_NAME,
@@ -150,7 +160,8 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
         logging.info(f"Created masterflat")
         del flats  # Deleting to free memory as we don't use flats anymore
 
-    raw_images: List[Path] = list(get_raw_images(NIGHT_INPUT_IMAGES_FOLDER))
+    raw_images: List[RawImageFile] = list(get_raw_images(NIGHT_INPUT_IMAGES_FOLDER))
+    image_duration = raw_images[0].image_duration()
     logging.info(f"Processing images")
     no_of_images_to_combine = config["processing"]["no_of_images_to_combine"]
     logging.info(f"Using no of images to combine: {no_of_images_to_combine}")
@@ -160,13 +171,16 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
     # Note the subtle typing difference between no_of_combined_images and no_of_images_to_combine
     no_of_combined_images = len(raw_images) // no_of_images_to_combine
 
+    log_files_to_normalize : List[LogFileCombinedFile] = []
+
     for i in range(no_of_combined_images):
         from_index = i * no_of_images_to_combine
         to_index = (i + 1) * no_of_images_to_combine
 
-        images_data = fit_data_from_fit_images(raw_images[from_index:to_index])
-        if len(crop_region) > 0:
-            images_data = [crop(matrix, rows, cols) for matrix in images_data]
+        images_data = [raw_image_file.data() for raw_image_file in raw_images[from_index:to_index]]
+        # Ensure that image dimensions are as specified by rows and cols 
+        # If there's extra noise cols or rows, we crop them 
+        images_data = [crop(matrix, rows, cols) for matrix in images_data]
 
         # Calibrate images
         images_data = calibrateImages(
@@ -185,8 +199,9 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
         aligned_images_data = []
         for index, image_data in enumerate(images_data):
             try:
-                aligned_images_data.append(imageAlignment(image_data, ref_image_path))
-            except Exception as e:
+                aligned_data, _ = image_alignment(image_data, ref_image_path)
+                aligned_images_data.append(aligned_data)
+            except Exception:
                 logging.error(f"Could not align image {raw_images[from_index + index]}")
                 logging.error(f"Skipping combination {from_index}-{to_index}")
                 break
@@ -199,38 +214,39 @@ def process_night(night: ConfigInputNight, config: Config, output: Path, night_d
             continue
 
         # Combination
-        combined_image_data = imageCombination(
-            aligned_images_data,
-            ALIGNED_COMBINED_OUTPUT_FOLDER
-            / f"m23_7.0-{(to_index // no_of_images_to_combine):03}.fit",
-            raw_images[from_index],  # Copy header from the first image in combination
-        )
+        combined_images_data = np.sum(aligned_images_data, axis=0)
+        sample_raw_image_file = raw_images[from_index]
+        aligned_combined_image_number = to_index // no_of_images_to_combine
+        aligned_combined_file_name = AlignedCombinedFile.generate_file_name(image_duration, aligned_combined_image_number)
+        aligned_combined_file = AlignedCombinedFile(ALIGNED_COMBINED_OUTPUT_FOLDER / aligned_combined_file_name)
+        aligned_combined_file.create_file(combined_images_data, sample_raw_image_file)
         logging.info(f"Combined images {from_index}-{to_index}")
 
         # Extraction
+        log_file_combined_file_name = LogFileCombinedFile.generate_file_name(night_date, aligned_combined_image_number, image_duration)
+        log_file_combined_file = LogFileCombinedFile(LOG_FILES_COMBINED_OUTPUT_FOLDER / log_file_combined_file_name)
         extract_stars(
-            combined_image_data,
-            ref_file_path,
-            save_as=LOG_FILES_COMBINED_OUTPUT_FOLDER
-            / f"{night_date.strftime(LOG_FILE_COMBINED_FILENAME_DATE_FORMAT)}_m23_7.0-{(to_index // 10):03}.txt",
-            radii_of_extraction=radii_of_extraction,
+            combined_images_data,
+            reference_log_file,
+            radii_of_extraction,
+            log_file_combined_file,
+            aligned_combined_file
         )
+        log_files_to_normalize.append(log_file_combined_file)
         logging.info(f"Extraction from combination {from_index}-{to_index} completed")
 
     # Normalization
     normalization_helper(
         radii_of_extraction,
         FLUX_LOGS_COMBINED_OUTPUT_FOLDER,
-        ref_file_path,
-        [
-            i
-            for i in LOG_FILES_COMBINED_OUTPUT_FOLDER.glob("*")
-            if i.is_file() and i.suffix == ".txt"
-        ],
+        reference_log_file,
+        log_files_to_normalize,
+        image_duration,
         night_date,
     )
 
 
+
 def start_data_processing_auxiliary(config: Config):
     """
     This function processes (one or more) nights defined in config dict by
